"""
A simple example, using list comprehension to filter odd numbers
"""

import doctest
from typing import List

from utils import compare_function_performances


def get_odd_numbers(start: int, end: int) -> List[int]:
    """
    >>> get_odd_numbers(0, 3)
    [1]

    >>> get_odd_numbers(3, 10)
    [3, 5, 7, 9]

    >>> get_odd_numbers(-3, 3)
    [-3, -1, 1]
    """
    return [
        n
        for n in range(start, end)
        if n % 2
    ]


def get_odd_numbers_naive(start: int, end: int) -> List[int]:
    """
    >>> get_odd_numbers_naive(0, 3)
    [1]

    >>> get_odd_numbers_naive(3, 10)
    [3, 5, 7, 9]

    >>> get_odd_numbers_naive(-3, 3)
    [-3, -1, 1]
    """
    odd_numbers = []
    for number in range(start, end):
        if number % 2:
            odd_numbers.append(number)
    return odd_numbers


if __name__ == '__main__':
    print(doctest.testmod())

    compare_function_performances(naive_method=get_odd_numbers_naive, your_method=get_odd_numbers, n_executions=10000, start=100, end=1000)
