"""
Here, we illustrate the power of the list comprehension other a simple and recurrent example: filtering a list
"""

import doctest
import random
from typing import List

from utils import compare_function_performances


class Image:
    def __init__(self, name: str, proba_is_cat: float):
        self.name: str = name
        self.proba_is_cat: float = proba_is_cat

    def __repr__(self):
        return f'<Image {self.name}>'


def filter_cat_images(images: List[Image], threshold: float = 0.5) -> List[Image]:
    """
    >>> images = [Image('cat01.png', 0.9), Image('cat02.png', 0.4), Image('dog01.png', 0.2), Image('dog02.png', 0.6)]
    >>> filter_cat_images(images)
    [<Image cat01.png>, <Image dog02.png>]

    >>> filter_cat_images(images, 0.4)
    [<Image cat01.png>, <Image cat02.png>, <Image dog02.png>]

    >>> filter_cat_images(images, 0.7)
    [<Image cat01.png>]

    >>> filter_cat_images([])
    []
    """
    # TODO
    return []


def filter_cat_image_naive(images: List[Image], threshold: float = 0.5) -> List[Image]:
    """
    >>> images = [Image('cat01.png', 0.9), Image('cat02.png', 0.4), Image('dog01.png', 0.2), Image('dog02.png', 0.6)]
    >>> filter_cat_image_naive(images)
    [<Image cat01.png>, <Image dog02.png>]

    >>> filter_cat_image_naive(images, 0.4)
    [<Image cat01.png>, <Image cat02.png>, <Image dog02.png>]

    >>> filter_cat_image_naive(images, 0.7)
    [<Image cat01.png>]

    >>> filter_cat_image_naive([])
    []
    """
    filtered_images = []
    for img in images:
        if img.proba_is_cat >= threshold:
            filtered_images.append(img)
    return filtered_images


if __name__ == '__main__':
    print(doctest.testmod())

    _images = [
        Image(f'random_img.png', random.random())
        for i in range(100)
    ]
    compare_function_performances(filter_cat_images, filter_cat_image_naive, images=_images, n_executions=100000)
