# List comprehension

An easy way to create lists in python.

## Syntax

```python
[
    expression(value)
    for value in iterable
    if condition(value)
]
```

## Example

Get the odd numbers, between two numbers.

```python
min, max = 10, 20
odd_number = [
    n
    for n in range(min, max)
    if n % 2
]
```

See [./0_odd_numbers.py] for details.

## Usages

- filtering
- transformation

## Pro / cons

### Pro

 - Faster, no need to create an empty list and add elements to it
   (list comprehension for odd numbers take 30% less time to perform than naive method)
 - Esay to read
 - pythonic
 
### Cons

 - Do not allow side effects, you need to create a expression function for that
 - You can't use the list until it is built (no intermediary list). Eg to build the list of prime members, it wouldn't be appropriate.

## Bonus

The syntax is also valid to construct sets or dictionaries.

### Set comprehension

We have a list of number, and we want to keep only the odd numbers, without duplicates

```python
raw_numbers = [1, 2, 4, 1, 4, 2, 5, 2, 5, 7, 8]
odd_numbers = {
    n
    for n in raw_numbers
    if n % 2
}
```


### Dict comprehension

We want to store the result of a function for a quick access in the future.

```python
def f(n):
    return n*n
    
numbers = [1, 2, 4, 8]

number2sqare = {
    n: f(n)
    for n in numbers
}
```

### Multiple iterations

You can even iterable multiple times in the same comprehension.

See [./00_multiple_iterations.py]

```python
list1 = [1, 2, 3]
list2 = [4, 10]
print([
    it1+it2
    for it1 in list1
    for it2 in list2
])
```

## More information

[python doc](https://docs.python.org/3/tutorial/datastructures.html#list-comprehensions)
