"""
Here we illustrate the simplicity of using zip, compared to having a indice variable.
"""

import doctest
import random
from typing import List

from utils import compare_function_performances


def filter_images(image_names: List[str], probabilities: List[float], threshold: float = 0.5) -> List[str]:
    """
    >>> image_names, probabilities = ['cat01.png', 'cat02.png', 'dog01.png', 'dog02.png'], [0.9, 0.4, 0.1, 0.6]
    >>> filter_images(image_names, probabilities)
    ['cat01.png', 'dog02.png']

    >>> filter_images(image_names, probabilities, threshold=0.4)
    ['cat01.png', 'cat02.png', 'dog02.png']

    >>> filter_images(image_names, probabilities, threshold=0.7)
    ['cat01.png']

    >>> filter_images([], [])
    []
    """
    # TODO
    return []


def filter_images_naive(image_names: List[str], probabilities: List[float], threshold: float = 0.5) -> List[str]:
    """
    >>> image_names, probabilities = ['cat01.png', 'cat02.png', 'dog01.png', 'dog02.png'], [0.9, 0.4, 0.1, 0.6]
    >>> filter_images_naive(image_names, probabilities)
    ['cat01.png', 'dog02.png']

    >>> filter_images_naive(image_names, probabilities, threshold=0.4)
    ['cat01.png', 'cat02.png', 'dog02.png']

    >>> filter_images_naive(image_names, probabilities, threshold=0.7)
    ['cat01.png']

    >>> filter_images_naive([], [])
    []
    """
    return [
        image_names[i]
        for i in range(len(image_names))
        if probabilities[i] >= threshold
    ]


if __name__ == '__main__':
    print(doctest.testmod())
    _image_names = ['test.png'] * 100
    _probabilities = [random.random() for _ in range(100)]
    compare_function_performances(filter_images, filter_images_naive, 100000, image_names=_image_names, probabilities=_probabilities)
