"""
Here, we want to illustrate the use of zip to get the best elements with the same indice, across multiple lists.
"""

import doctest
import random
from typing import List, Tuple

from utils import compare_function_performances


def find_most_probable_cat(image_names: List[str], probabilities: List[float]) -> Tuple[float, str]:
    """
    >>> image_names, probabilities = ['cat01.png', 'cat02.png', 'dog01.png', 'dog02.png'], [0.9, 0.4, 0.1, 0.6]
    >>> find_most_probable_cat(image_names, probabilities)
    (0.9, 'cat01.png')
    >>> find_most_probable_cat([], [])
    """
    if not len(image_names):
        return None

    # TODO
    return 0, None


def find_most_probable_cat_naive(image_names: List[str], probabilities: List[float]) -> Tuple[float, str]:
    """
    >>> image_names, probabilities = ['cat01.png', 'cat02.png', 'dog01.png', 'dog02.png'], [0.9, 0.4, 0.1, 0.6]
    >>> find_most_probable_cat_naive(image_names, probabilities)
    (0.9, 'cat01.png')
    >>> find_most_probable_cat_naive([], [])
    """
    n = len(image_names)
    if not n:
        return None

    best_name, best_proba = image_names[0], probabilities[0]
    for i in range(1, n):
        if best_proba < probabilities[i]:
            best_proba, best_name = probabilities[i], image_names[i]

    return best_proba, best_name


if __name__ == '__main__':
    print(doctest.testmod())

    _image_names = ['test.png'] * 10000
    _probabilities = [random.random() for _ in range(10000)]
    compare_function_performances(find_most_probable_cat, find_most_probable_cat_naive, 1000, image_names=_image_names, probabilities=_probabilities)
