# pathlib

pathlib is a python module that gives an easy way to deal with directories, files, ...

## Context

Opening and reading files with python can be really painful.

eg
```python
with open('/path/to/file/to/read', 'r') as f_read:
    content = f_read.read()
    
    with open('/path/to/file/to/write', 'w') as f_write:
        f_write.write(content)
        
```

(We needed the use of the `with` keyword to avoid keeping the file opened)

The pathlib library is a easier way to do that, among other things.

## Usage

### Ex 1

```python
from pathlib import Path

WORK_DIR = Path.home() / 'polystar'
path_file = WORK_DIR / 'data' / 'cats' / 'cat01.png'
image_data = path_file.read_text()  # Not the good way to do that, but it is an example :)
```

### Ex 2

See [./00_create_directory.py]

## Pros / Cons

### The Pros

 - Read/write a file `content = Path('/path/to/read').read_text()`
 - Move through directories `Path.home() / 'fold1' / 'fold2'` (no need to worry with the `/` anymore)
 - List a directory content (see [https://docs.python.org/3/library/pathlib.html#pathlib.Path.glob])
 
### The Cons

 - Not every module can handle a Path object (but more and more can), so your sometimes need to convert your Path to a str, using `str(your_path_obj)`
 
## More information

[See the module documentation](https://docs.python.org/3/library/pathlib.html)
